#!/usr/bin/perl
#Script pour administrer les listes sympa
#Auteur : SN-FBrosset
#Date création:17-03-2023
use lib split(/:/, $ENV{SYMPALIB} || ''), '/usr/share/sympa/lib';
use strict;
#use warnings;
use Term::ProgressBar 2.00;
use Digest::MD5;
use English qw(-no_match_vars);
use Fcntl qw();
use File::Basename qw();
use File::Copy qw();
use File::Path qw();
use Getopt::Long;
use Pod::Usage;
use POSIX qw();
use Data::Dumper;
use Net::LDAP;
use Sympa;
use Conf;
use Sympa::Robot;
use Sympa::Config_XML;
use Sympa::Constants;
use Sympa::DatabaseManager;
use Sympa::DatabaseDescription;
use Sympa::Family;
use Sympa::Language;
use Sympa::List;
use Sympa::Log;
use Sympa::Mailer;
use Sympa::Spindle::ProcessDigest;
use Sympa::Spindle::ProcessRequest;
use Sympa::Template;
use Sympa::Tools::Data;
use Sympa::Upgrade;
use Sympa::List::Config;
use Sympa::Config;
use Sympa::Scenario;

my %options;
GetOptions(\%main::options, 'listerModerateurs=s', 'verifierModerateurs=s', 'listerListes=s', 'stats=s');

my $default_lang;
my $robot;
my $language = Sympa::Language->instance;
my $mailer   = Sympa::Mailer->instance;
my $log = Sympa::Log->instance;
$robot = $ENV{SYMPA_DOMAIN};

_load();


$log->openlog($Conf::Conf{'syslog'}, $Conf::Conf{'log_socket_type'});


if ($main::options{'listerModerateurs'}) {
	recuperer_moderateurs($main::options{'listerModerateurs'});	
}

if ($main::options{'verifierModerateurs'}) {
        verifier_moderateurs($main::options{'verifierModerateurs'});
}

if ($main::options{'listerListes'}) {
        listerListes($main::options{'listerListes'});
}


if ($main::options{'stats'}) {
        recuperer_stats($main::options{'stats'});
}




sub recuperer_stats {
	#no warnings 'uninitialized';
	my $all_lists =  Sympa::List::get_lists($robot,
        filter => ['! status' => 'closed|family_closed']);
        my $last_message;
	my $chaineStats = "";
	my %listes = ();
	print "Récuparation des statistiques :\n";
	my $nb_listes = scalar @$all_lists;
	my $compteur = 0;
	my $progress = Term::ProgressBar->new($nb_listes);
        foreach my $list (@$all_lists) {
		++$compteur;
		#	print $list->{'name'};
		#if ($list->{'name'} eq 'test-fred1' || $list->{'name'} eq 'test-fred2' || $list->{'name'} eq 'test-fred3' || $list->{'name'} eq 'test-fred4' ){
			my %listeStats = ();
			my $listeName="";
			my $dateCreation;
			my $createur = "";
			my $emploieType = "";
			my $lastMessage;
			my $msgCount;
			my $msgTotal;
			my $total_Bouncing;
			my $FamilyName = "";
			my $owners="";
			my $editors="";
			my $scenarVisibility="";
			my $scenarSend="";
			my $bouncers="";
			my $chaine="";
			my $owner_listmaster="";
			my $owner_DtSrc_csv="";
			my $editor_DtSrc_csv="";
			my $member_DtSrc_csv="";
			my $owner_DtSrc_csv_detail="";
                        my $editor_DtSrc_csv_detail="";
                        my $member_DtSrc_csv_detail="";
			my $subscriber_DtSrc="";
			my $subscriber_DtSrc_Detail = "";
			my $subscriber_DtSrc_csv = "";
			my $liste_owner_DtSrc = "";
			my $liste_editor_DtSrc = "";
			my $liste_member_DtSrc = "";
			my %liste_subscriber_DtSrc = ();
			my $is_data_source_to = "";
			my $liste = Sympa::List->new($list->{'name'});
			#printf "%s\n",$list->{'name'};
			$listeName = $list->{'name'};
			$listeStats{"listeName"} = $listeName;
			$dateCreation = get_liste_date_cretation($list);
			$listeStats{"dateCreation"} = $dateCreation;
                	($createur, $emploieType) = get_createur($list);
			$listeStats{"createur"} = $createur;
			$listeStats{"emploieType"} = $emploieType;
			$last_message = get_last_message($list);
			$listeStats{"last_message"} = $last_message;
			$msgCount = $list->get_msg_count();
			$listeStats{"msgCount"} = $msgCount;
			$msgTotal = $list->get_total();
			$listeStats{"msgTotal"} = $msgTotal;
			$total_Bouncing = $list->get_total_bouncing();
			$listeStats{"total_Bouncing"} = $total_Bouncing;
			$FamilyName = get_family_name($list);
			$listeStats{"FamilyName"} = $FamilyName;
			$owner_listmaster = is_ownner_listmaster($list);	
			$listeStats{"owner_listmaster"} = $owner_listmaster;
			$owners = get_admins_csv($list,"owner");
			$listeStats{"owners"} = $owners;
			$editors = get_admins_csv($list,"editor");
			$listeStats{"editors"} = $editors;
			$scenarVisibility = get_scenario_visibility_name($list);
			$listeStats{"scenarVisibility"} = $scenarVisibility;
			$scenarSend = get_scenario_send_name($list);
			$listeStats{"scenarSend"} = $scenarSend;
			$bouncers = get_bouncers_csv($list);
			$listeStats{"bouncers"} = $bouncers;
			$liste_owner_DtSrc = recuperer_data_sources_owner($list);#get_data_sources_owner_csv($list);
                        $listeStats{"owner_DtSrc"} = $liste_owner_DtSrc;
			$owner_DtSrc_csv = get_data_sources_owner_csv($liste_owner_DtSrc, "non");
			$listeStats{"owner_DtSrc_csv"} = $owner_DtSrc_csv;
			$owner_DtSrc_csv_detail = get_data_sources_owner_csv($liste_owner_DtSrc, "oui");
			$listeStats{"owner_DtSrc_csv_Detail"} = $owner_DtSrc_csv_detail;
			$liste_editor_DtSrc = recuperer_data_sources_editor($list);# get_data_sources_editor_csv($list);
                        $listeStats{"editor_DtSrc"} = $liste_editor_DtSrc;
			$editor_DtSrc_csv = get_data_sources_editor_csv($liste_editor_DtSrc, 'non');
			$listeStats{"editor_DtSrc_csv"} = $editor_DtSrc_csv;
			$editor_DtSrc_csv_detail = get_data_sources_editor_csv($liste_editor_DtSrc, "oui");
                        $listeStats{"editor_DtSrc_csv_Detail"} = $editor_DtSrc_csv_detail;
			$liste_member_DtSrc = recuperer_data_sources_member($list);# get_data_sources_member_csv($list);
                        $listeStats{"member_DtSrc"} = $liste_member_DtSrc;
			$member_DtSrc_csv = get_data_sources_member_csv($liste_member_DtSrc, "non");
			$listeStats{"member_DtSrc_csv"} = $member_DtSrc_csv;
			$member_DtSrc_csv_detail = get_data_sources_member_csv($liste_member_DtSrc, "oui");
                        $listeStats{"member_DtSrc_csv_Detail"} = $member_DtSrc_csv_detail;
			%liste_subscriber_DtSrc = recuperer_data_sources_subscriber2($list);# get_data_sources_subscriber_csv2($list, 1);
			$listeStats{"subscriber_DtSrc"} = \%liste_subscriber_DtSrc;
			$subscriber_DtSrc_csv = get_data_sources_subscriber_csv2(\%liste_subscriber_DtSrc, 1);
			$listeStats{"subscriber_DtSrc_csv"} = $subscriber_DtSrc_csv;
			$subscriber_DtSrc_Detail = get_data_sources_subscriber_csv2(\%liste_subscriber_DtSrc, 0);
			$listeStats{"subscriber_DtSrc_Detail"} = $subscriber_DtSrc_Detail;
			$listeStats{"is_data_source_to"} = $is_data_source_to;
			$listes{$listeName} = \%listeStats;
			#}
		$progress->update($compteur);
        }
	$progress->update($nb_listes);
	analyser_liste_est_source_de_donnee(\%listes);
	ecrire_ListesStats(\%listes);
}

#Méthode qui récupère le nom du scénario visibility
sub get_scenario_visibility_name {
	my $list = shift(@_);
	my $result="";
	my $scenar_visibility_name = $list->{'admin'}{'visibility'}{'name'};
        my $scenario_visibility = Sympa::Scenario->new($list, 'visibility',
                  name => $scenar_visibility_name);
	#$result = $scenario_visibility->get_current_title();
	$result = $scenar_visibility_name;
	$result =~ tr/;/,/;
	return $result;
}

#Méthode qui récupère le nom du scénario send
sub get_scenario_send_name {
        my $list = shift(@_);
        my $result="";
        my $scenar_send_name = $list->{'admin'}{'send'}{'name'};
        my $scenario_send = Sympa::Scenario->new($list, 'send',
                  name => $scenar_send_name);
	#$result = $scenario_send->get_current_title();
	$result = $scenar_send_name;
        $result =~ tr/;/,/;
        return $result;
}

#Méthode qui vérifie si le propriétaire est le listmaster
sub is_ownner_listmaster {
	my $list = shift(@_);
	my $result="";
	my $admins = $list->get_admins("owner");
        if (scalar @$admins == 1 and @$admins[0]->{'email'} =~ /^\Qlistmaster\E/) {
                 $result = "oui";
        }
 	return $result;
}

#Méthode récupère les utilisateurs suivit un rôle
sub get_admins_csv {
	my ($list, $role) = @_;
	my $result="";
	my $admins = $list->get_current_admins;
  	foreach my $admin (@$admins) {
       		if ($admin->{'role'} eq $role) {
                	$result = $result . $admin->{'email'} . "\n";
                }
	}
	$result = "\"" . $result . "\"";
	return $result;
}

#Méthode qui récupère la famille de la liste
sub get_family_name {
	my $list = shift(@_);
	my $familyName = "";
	if (exists $list->{'admin'}{'family_name'}) {
        	$familyName = $list->{'admin'}{'family_name'};
        }
	return $familyName;
}

#Méthode qui récupère la date du dernier message envoyé
sub get_last_message {
	my $list = shift(@_);
	my $last_message = 0;
        if (open COUNT, $list->{'dir'} . '/msg_count') {
        	while (<COUNT>) {
        		$last_message = $1 if (/^(\d+)\s/ && ($1 > $last_message));
        	}	
        	close COUNT;
	}
        if ($last_message ne 0) {
        	$last_message = $language->gettext_strftime(
                "%d/%m/%Y", localtime($last_message * 86400));
        }
	return $last_message
}

#Méthode qui récupère la date de création de la liste
sub get_liste_date_cretation {
	my $list = shift(@_);
	return $language->gettext_strftime(
               "%d/%m/%Y",localtime($list->{'admin'}{'creation'}{'date_epoch'}));
}

#Méthode qui récupère l'emploi type du créateur de la liste
#d'après le champ employeeType du LDAP
sub get_createur {
	my $list = shift(@_);
	my $createur = $list->{'admin'}{'creation'}{'email'};
	my $emploieType = "";
         
	return ($createur, $emploieType);

}

#Méthode qui récupère le code erreur du bounce
sub recuprer_bounce_code {
	my $bounce = shift(@_);
	my $code = "";
	if ($bounce ne "") {
		my ($date1, $date2, $nb, $errorCode) = split / /, $bounce;
		$code = join(":", $nb, $errorCode);
	}
	return $code;
}

#Méthode écrit les statistiques dans un fichier
sub ecrire_ListesStats {
	my ($lstStats) = @_;
	my %listesStats = %$lstStats;
	my $chaineStats= "";
	foreach my $liste (keys %listesStats) {
		my $chaine = convertir_En_ChaineCsv($listesStats{$liste});
		$chaineStats = $chaineStats . $chaine . "\r\n";
	}
	my $headers = join(";","Nom","Date creation","createur","Type createur","Famille","Dernier message","Nb messages","Visiblite","Type envoi","Bounces","Listmaster","Proprietaires","Editeurs","Bouncers","Source de donnees Proprietaire", "Detail source de donnees Proprietaire","Source de donnees Moderateur","Detail source de donnees Moderateur", "Source de donnees Membre","Detail source de donnees Membre","Source de donnees Abonnees", "Details source de donnees Abonnees","La liste est une source de donnees de ");
	$chaineStats = $headers . "\r\n" . $chaineStats;
	ecrire_stats($chaineStats);
	
}

#Méthode qui concatène les élément d'une liste en chaine csv
sub convertir_En_ChaineCsv {
	my ($lstStats) = @_;
	my %listeStats = %$lstStats;
	my $chaine = join(";",$listeStats{"listeName"},$listeStats{"dateCreation"},$listeStats{"createur"},$listeStats{"emploieType"},$listeStats{"FamilyName"},$listeStats{"last_message"},$listeStats{"msgCount"},$listeStats{"scenarVisibility"},$listeStats{"scenarSend"},$listeStats{"total_Bouncing"},$listeStats{"owner_listmaster"},$listeStats{"owners"},$listeStats{"editors"},$listeStats{"bouncers"},$listeStats{"owner_DtSrc_csv"},$listeStats{"owner_DtSrc_csv_Detail"},$listeStats{"editor_DtSrc_csv"},$listeStats{"editor_DtSrc_csv_Detail"},$listeStats{"member_DtSrc_csv"},$listeStats{"member_DtSrc_csv_Detail"},$listeStats{"subscriber_DtSrc_csv"},$listeStats{"subscriber_DtSrc_Detail"},$listeStats{"is_data_source_to"});
	return $chaine;

}

sub ecrire_stats {
	my $stats = shift(@_);
	my $filename = '/opt/outils/stats.csv';

	open(FH, '>', $filename) or die $!;

	print FH $stats;

	close(FH);
}

sub ecrire_csv {
         my ($fichier, $chaine) = @_;

         open(FH, '>', $fichier) or die $!;

         print FH $chaine;

         close(FH);
}

#Méthode qui vérifie si la liste est elle-même une source de donnée
#d'une autre liste
sub analyser_liste_est_source_de_donnee {
	my ($listes) = @_;
   	my %listes_ref = %$listes;
	my $is_data_source_to = "";
	my $nb_listes = scalar keys %listes_ref;
	my $compteur = 0;
	printf "Annalyse des sources de données\n";
	my $progress = Term::ProgressBar->new($nb_listes);
	$progress->minor(0);
	my $next_update = 0;
	foreach my $listeSource (keys %listes_ref) {
		++$compteur;
		$is_data_source_to = "";
		foreach my $liste (keys %listes_ref) {
			if ($listeSource ne $liste) {
				if (defined $listes_ref{$liste}{'editor_DtSrc'}){
					my $test = $listes_ref{$liste}{'editor_DtSrc'};
					foreach my $dataSrc (@$test){
						if ($dataSrc->{'source'} eq 'liste') {
							if (defined $dataSrc->{'source_parameters'}) {
								if ($dataSrc->{'source_parameters'} eq $listeSource) {
									$is_data_source_to = $is_data_source_to . $liste . ":editors\n";
								}
							}
						}
					}
				}

				if (defined $listes_ref{$liste}{'owner_DtSrc'}){
                                         my $test = $listes_ref{$liste}{'owner_DtSrc'};
                                         foreach my $dataSrc (@$test){
                                                 if ($dataSrc->{'source'} eq 'liste') {
							if (defined $dataSrc->{'source_parameters'}) {
								if ($dataSrc->{'source_parameters'} eq $listeSource) {
						 			$is_data_source_to = $is_data_source_to . $liste . ":owners\n";
								}
							}	
                                                 }
                                         }
                                 }

				 if (defined $listes_ref{$liste}{'member_DtSrc'}){
                                         my $test = $listes_ref{$liste}{'member_DtSrc'};
                                         foreach my $dataSrc (@$test){
                                                 if ($dataSrc->{'source'} eq 'liste') {
							if (defined $dataSrc->{'source_parameters'}) {
								if ($dataSrc->{'source_parameters'} eq $listeSource) {
						 			$is_data_source_to = $is_data_source_to . $liste . ":memberss\n";
								}
							}
                                                 }
                                         }
                                 }

				if (defined $listes_ref{$liste}{'subscriber_DtSrc'}{'include_sympa_list'}){
					my $test = $listes_ref{$liste}{'subscriber_DtSrc'}{'include_sympa_list'};
					foreach my $dataSrc (@$test){
						if ($dataSrc->{'listname'} eq $listeSource) {
							$is_data_source_to = $is_data_source_to . $liste . ":subscribers\n";
						}

					}
	
				}

			}

		}
		$progress->update($compteur);# if $compteur >= $next_update;
		$is_data_source_to = "\"" . $is_data_source_to . "\"";
		$listes_ref{$listeSource}{'is_data_source_to'} = $is_data_source_to;
        }
	$progress->update($nb_listes);
}

#Méthode qui récupère en format chaine csv les sources de données des propriétaires  
sub get_data_sources_owner_csv {
	my ($owner_DataSources,$detail) = @_;
	my $owner_DtSrc="";
        foreach my $owner_DataSource (@$owner_DataSources) {
		if (defined $owner_DataSource->{'source'}) {
			if ($detail eq "non") {
                        	$owner_DtSrc = $owner_DtSrc . $owner_DataSource->{'source'} . "\n";
			} else {
				if (defined $owner_DataSource->{'source_parameters'}) {
                                       $owner_DtSrc =  $owner_DtSrc . $owner_DataSource->{'source'} . " : " . $owner_DataSource->{'source_parameters'} . "\n";
                                } else {
					$owner_DtSrc =  $owner_DtSrc . $owner_DataSource->{'source'} . " : " . "vide\n";
				}
                        }
                }

		#$owner_DtSrc = $owner_DtSrc . $owner_DataSource->{'source'} . "\n";
        }
       	$owner_DtSrc = "\"" . $owner_DtSrc . "\"";
	return $owner_DtSrc;
}

#Méthode qui récupère les sources de données des propriétaires
sub recuperer_data_sources_owner {
	my $list = shift(@_);
	my $config = Sympa::List::Config->new($list, config => $list->{'admin'});
	return $config->get('owner_include');

}

#Méthode qui récupère en format chaine csv les sources de données des éditeurs
sub get_data_sources_editor_csv {
	my ($editor_DataSources,$detail) = @_;
	my $editor_DtSrc = "";
        foreach my $editor_DataSource (@$editor_DataSources) {
        	if (defined $editor_DataSource->{'source'}) {
			if ($detail eq "non") {
                        	$editor_DtSrc = $editor_DtSrc . $editor_DataSource->{'source'} . "\n";
			} else {
				if (defined $editor_DataSource->{'source_parameters'}) {
                                       $editor_DtSrc =  $editor_DtSrc . $editor_DataSource->{'source'} . " : " . $editor_DataSource->{'source_parameters'} . "\n";
                                } else {
				$editor_DtSrc =  $editor_DtSrc . $editor_DataSource->{'source'} . " : " . "vide\n";
				}
                        }
                }
		#$editor_DtSrc = $editor_DtSrc . $editor_DataSource->{'source'} . "\n";
        }
        $editor_DtSrc = "\"" . $editor_DtSrc . "\"";
	return $editor_DtSrc;
}

#Méthode qui récupère les sources de données des éditeurs
sub recuperer_data_sources_editor {
        my $list = shift(@_);
        my %data_sources = ();
	my $config = Sympa::List::Config->new($list, config => $list->{'admin'});
	return $config->get('editor_include');
}

#Méthode qui récupère en format chaine csv les sources de données des memebres
sub get_data_sources_member_csv {
	my ($member_DataSources,$detail) = @_;
	my $member_DtSrc = "";
        foreach my $member_DataSource (@$member_DataSources) {
		if (defined $member_DataSource->{'source'}) {
			if ($detail eq "non") {
				$member_DtSrc = $member_DtSrc . $member_DataSource->{'source'} . "\n";
			} else {
				if (defined $member_DataSource->{'source_parameters'}) {
					$member_DtSrc = $member_DtSrc . $member_DataSource->{'source'} . " : " . $member_DataSource->{'source_parameters'} . "\n";
				} else {
					$member_DtSrc = $member_DtSrc . $member_DataSource->{'source'} . " : " . "vide\n";
				}	
			}
		}
        }
	$member_DtSrc = "\"" . $member_DtSrc . "\"";
	return $member_DtSrc;
}

#Méthode qui récupère les sources de données des memebres
sub recuperer_data_sources_member {
        my $list = shift(@_);
        my $config = Sympa::List::Config->new($list, config => $list->{'admin'});
        return $config->get('member_include');

}

#Méthode qui récupère en format chaine csv les sources de données des abonnées
sub get_data_sources_subscriber_csv2 {
	my ($subscriber_DataSrc, $detail) = @_;
	my %subscriber_DataSources = %$subscriber_DataSrc;
	my $subscriber_DtSrc = "";
	for(keys %subscriber_DataSources) {
		
		my $data_source_params = $subscriber_DataSources{$_};
                 foreach my $param (@$data_source_params) {
			if ($_ eq 'include_file') {
				$subscriber_DtSrc = $detail ? $subscriber_DtSrc . $_ . "\n" :  
				$subscriber_DtSrc . $_ . "\n" . "=>" . "fichier:" . $param . "\n";
			}

			if ($_ eq 'include_remote_file') {
				$subscriber_DtSrc = $detail ? $subscriber_DtSrc . $_ . "\n" : 
				$subscriber_DtSrc . $_ . "\n" . "=>" . "url:" . $param->{'url'} . "\n";
                        }

			if ($_ eq 'include_sympa_list') {
				$subscriber_DtSrc = $detail ? $subscriber_DtSrc . $_ . "\n" : 
				$subscriber_DtSrc . $_ . "\n" . "=>" . "Listname:" . $param->{'listname'} . "\n"
			}

	                if ($_ eq 'include_remote_sympa_list') {
				$subscriber_DtSrc = $detail ? $subscriber_DtSrc . $_ . "\n" : 
				$subscriber_DtSrc . $_ . "\n" . "=>" . "url:" . $param->{'url'} . "\n";
                        }

			if ($_ eq 'include_ldap_query') {
				$subscriber_DtSrc = $detail ? $subscriber_DtSrc . $_ . "\n" : 
				$subscriber_DtSrc . $_ . "\n" . "=>" . "filter:" . $param->{'filter'} . "\n";
                        }

			if ($_ eq 'include_ldap_2level_query') {
				$subscriber_DtSrc = $detail ? $subscriber_DtSrc . $_ . "\n" : 
				$subscriber_DtSrc . $_ . "\n" . "=>" . "filter1:" . $param->{'filter1'} . "\n" . "=>" . "attrs2:" . $param->{'attrs2'} . "\n";
                        }

			if ($_ eq 'include_sql_query') {
				$subscriber_DtSrc = $subscriber_DtSrc . $_ . "\n";
                        }
                 }
         }
        $subscriber_DtSrc = "\"" . $subscriber_DtSrc . "\"";
	return $subscriber_DtSrc;
}

#Méthode qui récupère les sources de données des abonnées
sub recuperer_data_sources_subscriber2 {
        my $list = shift(@_);
        my %data_sources = ();
        my $config = Sympa::List::Config->new($list, config => $list->{'admin'});
       	
	if (defined $config->get('include_file')){
               $data_sources{'include_file'} =  $config->get('include_file');
        }

	if (defined $config->get('include_remote_file')){
               $data_sources{'include_remote_file'} =  $config->get('include_remote_file');
        }

	if (defined $config->get('include_sympa_list')){
               $data_sources{'include_sympa_list'} =  $config->get('include_sympa_list');
        }

	if (defined $config->get('include_remote_sympa_list')){
               $data_sources{'include_remote_sympa_list'} =  $config->get('include_remote_sympa_list');
        }

	if (defined $config->get('include_ldap_query')){
                $data_sources{'include_ldap_query'} =  $config->get('include_ldap_query');
        }

	if (defined $config->get('include_ldap_2level_query')){
                $data_sources{'include_ldap_2level_query'} =  $config->get('include_ldap_2level_query');
        }

	if (defined $config->get('include_sql_query')){
                $data_sources{'include_sql_query'} =  $config->get('include_sql_query');
        }

	return %data_sources;
}

#Méthode qui récupère en format chaine csv les bounces
#et qui loggue les bouncers en erreur dans un fichier texte
sub get_bouncers_csv {
	my $list = shift(@_);
	my $bouncers = "";
	my $listeBouncers = recuperer_bouncers($list);
	if ((scalar @$listeBouncers) > 0) {
		foreach my $bouncer (@$listeBouncers) {
                	$bouncers = $bouncers . $bouncer->{'email'} . "\n";
        	}	
        	$bouncers = "\"" . $bouncers . "\"";
        	if (length($bouncers) > 30000) {
        		$bouncers = "Nombre de bouncers : " . scalar @$listeBouncers;
        	}
	}	
	return $bouncers;
}


#Méthode qui récupère les bouncers
sub recuperer_bouncers {
	my $liste = shift(@_);
	my $sdm = Sympa::DatabaseManager->instance;
    	my $limit = $options{limit};
    	my $offset = $options{offset};
    	my $order = $options{order};
    	my $cond = $options{othercondition};
	my $sth;
	unless (
        	$sdm
        	and $sth = $sdm->do_prepared_query(
            	sprintf(
                q{SELECT %s
                FROM subscriber_table
                WHERE list_subscriber = ? AND robot_subscriber = ? AND
                      bounce_subscriber IS NOT NULL},
                _list_member_cols($sdm)
            ),
            $liste->{'name'},
            $liste->{'domain'}
        )) {
        	print "Unable to get bouncing users";
        	return undef;
	}
	my $bouncers = $sth->fetchall_arrayref({}) || [];
    	$sth->finish;
	
	return $bouncers;
}

sub _list_member_cols {
    my $sdm = shift;

    my %map_field = _map_list_member_cols();
    return join ', ', map {
        my $col = $map_field{$_};
        ($col eq $_) ? $col : sprintf('%s AS "%s"', $col, $_);
    } sort keys %map_field;
}

sub _map_list_member_cols {
    my %map_field = (
        date        => 'date_epoch_subscriber',
        update_date => 'update_epoch_subscriber',
        gecos       => 'comment_subscriber',
        email       => 'user_subscriber',
        startdate   => 'suspend_start_date_subscriber',
        enddate     => 'suspend_end_date_subscriber',
	bounce      => 'bounce_subscriber',
    );

    my $fields =
        {Sympa::DatabaseDescription::full_db_struct()}->{'subscriber_table'}
        ->{fields};
    foreach my $f (keys %$fields) {
        next if $f eq 'list_subscriber' or $f eq 'robot_subscriber';

        my $k = {reverse %map_field}->{$f};
        unless ($k) {
            $k = $f;
            $k =~ s/_subscriber\z//;
            $map_field{$k} = $f;
        }
    }
    # Additional DB fields.
    if ($Conf::Conf{'db_additional_subscriber_fields'}) {
        foreach my $f (split /\s*,\s*/,
            $Conf::Conf{'db_additional_subscriber_fields'}) {
            $map_field{$f} = $f;
        }
    }

    return %map_field;
}

#Méthode qui récupère les modérateurs
sub recuperer_moderateurs {
	
	my $liste = shift(@_);
	print $liste;
	my $list = Sympa::List->new($liste);

	unless ($list) {
        	printf STDERR "Incorrect list name %s\n", $liste ;
        	exit 1;
    	}

	my $liste_users = $list->get_admins("editor");

	foreach my $user (@$liste_users) {
        	#print Dumper($user);
        	printf "%s\n",$user->{'email'};
	}	
}

sub listerListes {
	my $famille = shift(@_);
	my $family = Sympa::Family->new($famille, "prelistes.insa-lyon.fr");
	my $listes = Sympa::List::get_lists($family);
	unless ($listes) {
                printf STDERR "Aucunes listes" ;
                exit 1;
        }


 	foreach my $list (@$listes) {
                #print Dumper($user);
                printf "%s\n",$list->{'name'};
        }


}



# Load configuration.
sub _load {
    ## Load sympa.conf.
    unless (Conf::load(Conf::get_sympa_conf(), 'no_db')) {    #Site and Robot
        die sprintf
            "Unable to load sympa configuration, file %s or one of the vhost robot.conf files contain errors. Exiting.\n",
            Conf::get_sympa_conf();
    }

    ## Open the syslog and say we're read out stuff.
    $log->openlog($Conf::Conf{'syslog'}, $Conf::Conf{'log_socket_type'});

    # Enable SMTP logging if required
    $mailer->{log_smtp} = $main::options{'mail'}
        || Sympa::Tools::Data::smart_eq($Conf::Conf{'log_smtp'}, 'on');

    # setting log_level using conf unless it is set by calling option
    if (defined $main::options{'log_level'}) {
        $log->{level} = $main::options{'log_level'};
        $log->syslog(
            'info',
            'Configuration file read, log level set using options: %s',
            $main::options{'log_level'}
        );
    } else {
        $log->{level} = $Conf::Conf{'log_level'};
        $log->syslog(
            'info',
            'Configuration file read, default log level %s',
            $Conf::Conf{'log_level'}
        );
    }


    # Check database connectivity.
    unless (Sympa::DatabaseManager->instance) {
        die sprintf
            "Database %s defined in sympa.conf is unreachable. verify db_xxx parameters in sympa.conf\n",
            $Conf::Conf{'db_name'};
    }

    # Now trying to load full config (including database)
    unless (Conf::load()) {    #FIXME: load Site, then robot cache
        die sprintf
            "Unable to load Sympa configuration, file %s or any of the virtual host robot.conf files contain errors. Exiting.\n",
            Conf::get_sympa_conf();
    }

    ## Set locale configuration
    ## Compatibility with version < 2.3.3
    $main::options{'lang'} =~ s/\.cat$//
        if defined $main::options{'lang'};
    $default_lang =
        $language->set_lang($main::options{'lang'}, $Conf::Conf{'lang'},
		        'en');

    ## Main program
    if (!chdir($Conf::Conf{'home'})) {
        die sprintf 'Can\'t chdir to %s: %s', $Conf::Conf{'home'}, $ERRNO;
        ## Function never returns.
    }

    ## Check for several files.
    unless (Conf::checkfiles_as_root()) {
        die "Missing files\n";
    }
}



exit(0);

